// Datei: I2CLCD.C
// 
// Ansteuerung einer I2C-LCD-Anzeige
// z.B. LBN223R
//
// hk@holger-klabunde.de 
// www.holger-klabunde.de
// 23.03.2002
// Compiler CC5x

#include "prozess.h"  // Der verwendete Prozessor
#include "protos.h"
#include "lbntest.h"
#include "i2c.h"

//*******************************************
// Schreibt ein Steuerbyte in die LCD-Anzeige
//*******************************************
void LCDWriteRaw(unsigned char data)
{
 I2C_START(); //Startbedingung senden,enthlt I2C_STOP()!!
 I2C_Write(LBN223); //I2C-Adresse und SchreibBit senden
 I2C_Write(0x00);
 I2C_Write(data); //Daten senden
 I2C_STOP();
 Delay100us(2);
}

//*******************************************
// Schreibt ein Datenbyte in die LCD-Anzeige
//*******************************************
void LCDWriteByte(unsigned char data)
{
 unsigned char by;  //Zwischenergebnisse

 by=data;
 //Daten eventuell umsetzen auf Sonderzeichen
 //im Character RAM
#ifdef EUROPA
 switch(by)
  {
   case 0xE1: by=0x84; break; // ''
   case 0xEF: by=0x94; break; // ''
   case 0xF5: by=0x81; break; // ''
   case 0xE2: by=0xE0; break; // ''
   case 0xB5: by=0x8E; break; // ''
//   case 0xB5: by=0x00; break; // '' selbstdefiniert sieht besser aus !
   case 0xB6: by=0x99; break; // ''
   case 0xB7: by=0x9A; break; // ''
   case 0xFC: by=0x06; break; // Euro
   case 0xE4: by=0xEA; break; // mycro
   case 0xDE: by=0x7E; break; // '~' 
   case 0xDF: by=0x07; break; // '' 
   case 0xFB: by=0x05; break; // '\'
   default: break;
  }
#endif

#ifdef INTERNATIONAL
 switch(by)
  {
   case 0xB5: by=0x00; break; // ''
   case 0xB6: by=0x01; break; // ''
   case 0xB7: by=0x02; break; // ''
   case 0xFB: by=0x05; break; // '\'
   case 0xFC: by=0x06; break; // Euro
   default: break;
  }
#endif

 I2C_Write(by); //Daten senden
 Delay100us(2);
}

//**************************************************
//Schreibt einen Buchstaben in die LCD-Anzeige
//**************************************************
void LCDWriteChar(char ch)
{
 I2C_START(); //Startbedingung senden,enthlt I2C_STOP()!!
 I2C_Write(LBN223); //I2C-Adresse und SchreibBit senden
 I2C_Write(0x40);
 LCDWriteByte(ch);
 I2C_STOP();
 Delay100us(2);
}

//**************************************************
//Schreibt einen Text in die LCD-Anzeige
//**************************************************
void LCDWrite(const char *__putstr)
{
 char __ps;

 I2C_START(); //Startbedingung senden,enthlt I2C_STOP()!!
 I2C_Write(LBN223); //I2C-Adresse und SchreibBit senden
 I2C_Write(0x40);

 do
  {
   __ps = *__putstr;
   __putstr++;

   if (__ps!=0) LCDWriteByte(__ps);

  }while(__ps!=0);

 I2C_STOP();
 Delay100us(2);
}

//****************************************************
// Springt auf Zeile 'x' Position 'y' der LCD-Anzeige 
//****************************************************
void LCDPos(unsigned char x,unsigned char y)
{
 unsigned char result;  //Zwischenergebnisse
// 0x80 wegen Display Adress Set.
// Erste Position ist bei mir 1, nicht 0.
// Offset's 1x8, 1x20, 1x24, 1x40, 2x16, 2x20, 2x40
// Zeile 1 -> 0x00+0x80-0x01=0x7F
// Zeile 2 -> 0x40+0x80-0x01=0xBF 
// 1x16 Displays sind 2x8 Displays !!!

 switch(x)
  {
   case 1: result=0x7F+y; break;
   case 2: result=0xBF+y; break;
   default : break;
  }

 LCDWriteRaw(result); 
}

//*****************************************
// Cursor einschalten, Cursor blinkt nicht
//*****************************************
void LCDCursorOn()
{ LCDWriteRaw(0x0E); Delay100us(2); }

//*****************************************
// Cursor einschalten, Cursor blinkt 
//*****************************************
void LCDCursorBlink()
{ LCDWriteRaw(0x0F); Delay100us(2); }

//*****************************************
// Cursor ausschalten
//*****************************************
void LCDCursorOff()
{ LCDWriteRaw(0x0C); Delay100us(2); }

//*****************************************
// Lscht die LCD Anzeige
// Cursor steht dann auf Position 1,Zeile 1
//*****************************************
void LCDCls()
{ LCDWriteRaw(0x01); Delay1ms(5); }

//*********************************************
//Stellt die Funktionsweise der LCD Anzeige ein
//*********************************************
void LCDInit()
{
 I2C_START(); //Startbedingung senden,enthlt I2C_STOP()!!

 I2C_Write(LBN223); //I2C-Adresse und SchreibBit senden
// if(I2CERR>0) { return; }

 I2C_Write(0x00);
 Delay100us(2);
// if(I2CERR>0) { return; }

 I2C_Write(0x2A);
 Delay100us(2);
// if(I2CERR>0) { return; }

 I2C_Write(0x0E);
 Delay100us(2);
// if(I2CERR>0) { return; }

 I2C_Write(0x06);
 Delay100us(2);
// if(I2CERR>0) { return; }

 I2C_STOP();
}
